define(['durandal/http', 'durandal/app'], function (http, app) {
    
    return {
        data: ko.observableArray([]),
        buttonText: "Update!",
        manager: null,
        labelText: "Hit the button to update the model 'desc' value of the first record with the value of 'some custom description & watch out the grid!",
        buttonClick: function () {
            // update data (the model for the IGnite Grid)
            this.data()[0].Description("some custom description");
        },
        commitClick: function () {
            var transactions = $("#grid1").igGrid("pendingTransactions"), self = this;
            transactions.forEach(function (t) {
                if (t.type == "newrow") {
                    // create an entity using Breeze's API
                    self.manager.createEntity('TodoItem', t.row);
                } else if (t.type == "deleterow") {
                    // find the entity
                    var entity = self.manager.getEntityByKey("TodoItem", t.rowId);
                    entity.entityAspect.setDeleted();
                } else {
                    // update of an existing entity
                    // 1. find the entity
                    var entity = self.manager.getEntityByKey("TodoItem", t.rowId);
                    // 2. fill new values
                    //entity.Id(t.row.Id);
                    entity.Description(t.row.Description);
                    entity.CreatedAt(t.row.CreatedAt);
                    entity.IsDone(t.row.IsDone);
                    // 3. update is done automatically (internally) by Breeze / ko, as soon as you modify a prop
                    // so we don't need to set any state manually
                }
            });
            this.manager.saveChanges().then(function (saveResult) {
                console.log("# of Todos saved = " + saveResult.entities.length);
                // now we need to clear the grid's transactions and add the items to the data array
                // via knockout, which will also update the grid itself
                // we do this so that we hijack the grid's updating, by telling it not to keep any transaction and dirty UI state,
                //we will do this through breeze and knockout
                $("#grid1").igGrid("rollback");
                console.log("performed rollback");
                saveResult.entities.forEach(function (item) {
                    var state = item.entityAspect.entityState;
                    if (state.isDetached()) {
                        // delete it from self.data
                        for (var i = 0; i < self.data().length; i++) {
                            if (self.data()[i].Id() == item.Id()) {
                                self.data.remove(self.data()[i]);
                                break;
                            }
                        }
                        // unfortunately Breeze treats modified and added entities as "Unchanged"
                        // so we need to differentiate between an update and a new add by ourselves
                    } else if (state.isAdded() || state.isModified() || state.isUnchanged()) {
                        var found = false;
                        // find the item in self.data, and modify the values
                        for (var i = 0; i < self.data().length; i++) {
                            if (self.data()[i].Id() == item.Id()) {
                                found = true;
                                // set new values
                                var currentItem = self.data()[i];
                                //currentItem.Id(item.Id());
                                currentItem.Description(item.Description());
                                currentItem.CreatedAt(item.CreatedAt());
                                currentItem.IsDone(item.IsDone());
                                break;
                            }
                        }
                        // Added
                        if (!found) {
                            self.data.push({
                                Id: item.Id,
                                Description: item.Description,
                                CreatedAt: item.CreatedAt,
                                IsDone: item.IsDone
                            }); // the rows in the grid will be added automatically
                        }
                    } 
                });
            }).fail(function (error) {
                console.log("Save failed: " + error.message);
            });
        },
        commitText: "Click to Save Changes using Breeze Entity Manager. the grid's transactions are read and passed to Breeze's API",
        commitButtonText: "Save Changes",
        activate: function () {
            //the router's activator calls this function and waits for it to complete before proceding
            if (this.data().length > 0) {
                return;
            }
            function Item(id, desc, createdAt, isDone) {
                return {
                    // we don't have to wrap with ko.observable() because it's already done by Breeze ! -; ) 
                    Id: id,
                    Description: desc,
                    CreatedAt: createdAt,
                    IsDone: isDone
                };
            };
            this.manager = new breeze.EntityManager("/api/todos");
            var query = breeze.EntityQuery
               .from("Todos")
               .orderBy("CreatedAt");
            var self = this;
            return this.manager.executeQuery(query).then(function (data) {
                data.results.forEach(function (item) {
                    self.data().push(new Item(item.Id, item.Description, item.CreatedAt, item.IsDone));
                });
            });
        },
        canDeactivate: function () {
            //the router's activator calls this function to see if it can leave the screen
            return app.showMessage('Are you sure you want to leave this page?', 'Navigate', ['Yes', 'No']);
        }
    };
});